package com.jfbrother.util;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import com.alibaba.fastjson.JSONArray;

/**
 * 参数接收工具类
 * 
 * @author luosz
 *
 */
public class RecieveUtils {
	/**
	 * 根据需要接收的参数名，接受参数，转化为map对象
	 * 
	 * @param request
	 *            http请求
	 * @param args
	 *            需要接收的参数名
	 * @author luosz
	 * @return
	 */
	public static Map<String, Object> getParams(HttpServletRequest request, String... args) {
		Map<String, Object> result = new HashMap<String, Object>();
		// 如果没有传入参数名，相当于不需要接收，直接返回一个空的map
		if (args == null) {
			return result;
		}
		// 遍历参数名，根据参数名从request中获取具体的参数值，存入map对象
		for (String arg : args) {
			if (arg == null || "".equals(arg)) {
				continue;
			}

			String[] attrs = request.getParameterValues(arg);
			if (attrs == null || attrs.length == 0) {
				//如果为null，不能主动设置为null，否则会自动转化为null字符串。不设置该值即可
				//result.put(arg, null);
			} else if (attrs.length == 1) {
				result.put(arg, attrs[0]);
			} else {
				JSONArray array=new JSONArray();
				for(String attr:attrs){
					array.add(attr);
				}
				result.put(arg, array.toJSONString());
			}
			
		}

		return result;
	}

	/**
	 * 根据需要接收的参数名，接受参数，转化为map对象
	 * 
	 * @param request
	 *            http请求
	 * @param listField
	 *            需要接收的参数名
	 * @author luosz
	 * @return
	 */
	public static Map<String, Object> getParams(HttpServletRequest request, List<String> listField) {
		return getParams(request, listField.toArray(new String[listField.size()]));
	}

	/**
	 * 根据需要接收的参数名，接受参数，转化为bean
	 * 
	 * @param request
	 *            http请求
	 * @param clazz
	 *            泛型，bean的class类型
	 * @param listField
	 *            需要接收的参数名
	 * @author luosz
	 * @return
	 */
	public static <T> T getBean(HttpServletRequest request, Class<T> clazz, List<String> listField) {
		return ConvertUtils.mapToBean(getParams(request, listField.toArray(new String[listField.size()])), clazz);
	}

	/**
	 * 根据需要接收的参数名，接受参数，转化为bean
	 * 
	 * @param request
	 *            http请求
	 * @param clazz
	 *            泛型，bean的class类型
	 * @param args
	 *            需要接收的参数名
	 * @author luosz
	 * @return
	 */
	public static <T> T getBean(HttpServletRequest request, Class<T> clazz, String... args) {
		return ConvertUtils.mapToBean(getParams(request, args), clazz);
	}

	/**
	 * 自动根据bean的属性，从request里面获取数据并填充
	 * 
	 * @param request
	 *            http请求
	 * @param clazz
	 *            泛型，bean的class类型
	 * @author luosz
	 * @return
	 */
	public static <T> T getBean(HttpServletRequest request, Class<T> clazz) {
		// 或许bean文件并根据bean文件获得所有get，set方法，根据set方法获取到所有属性名
		List<String> listField = new ArrayList<String>();
		for (Method m : clazz.getDeclaredMethods()) {
			String methodName = m.getName();
			if (methodName.startsWith("set")) {
				int length = "set".length();
				String fieldName = methodName.substring(length, length + 1).toLowerCase()
						+ methodName.substring(length + 1);
				listField.add(fieldName);
			}
		}
		return ConvertUtils.mapToBean(getParams(request, listField), clazz);
	}

	/**
	 * 根据需要接收的参数名，接受参数，转化为map对象
	 * 
	 * @param args
	 *            需要接收的参数名
	 * @author luosz
	 * @return
	 */
	public static Map<String, Object> getParams(String... args) {
		return getParams(RequestUtils.getRequest(), args);
	}

	/**
	 * 根据需要接收的参数名，接受参数，转化为map对象
	 * 
	 * @param listField
	 *            需要接收的参数名
	 * @author luosz
	 * @return
	 */
	public static Map<String, Object> getParams(List<String> listField) {
		return getParams(RequestUtils.getRequest(), listField.toArray(new String[listField.size()]));
	}

	/**
	 * 根据需要接收的参数名，接受参数，转化为bean
	 * 
	 * @param clazz
	 *            泛型，bean的class类型
	 * @param listField
	 *            需要接收的参数名
	 * @author luosz
	 * @return
	 */
	public static <T> T getBean(Class<T> clazz, List<String> listField) {
		return getBean(RequestUtils.getRequest(), clazz, listField);
	}

	/**
	 * 根据需要接收的参数名，接受参数，填充到bean
	 * 
	 * @param clazz
	 *            泛型，bean的class类型
	 * @param args
	 *            需要接收的参数名
	 * @author luosz
	 * @return
	 */
	public static <T> T getBean(Class<T> clazz, String... args) {
		return getBean(RequestUtils.getRequest(), clazz, args);
	}

	/**
	 * 自动根据bean的属性，从request里面获取数据并填充
	 * 
	 * @param clazz
	 * @return
	 */
	public static <T> T getBean(Class<T> clazz) {
		return getBean(RequestUtils.getRequest(), clazz);
	}
}
